/*   Библиотека для работы с e24, в основном для LabView (в традиционных языках
 * программирования это было бы сделать проще).
 */
#include <stdio.h>
#include <inttypes.h>
#include <unistd.h>
#include <string.h>
#include <sys/time.h>
#include <signal.h>
#include "vhid.h"
#include "memfile.h"
#include "../../firmware/commands.h"
#include "e24hid.h"

const float E24_MAX_RANGE = 2.5;

/* Переинициализация АЦП e24 и установка настроек по умолчанию
 * 
 * Входные параметры:
 *   нет
 * Результат:
 *   0 - успех
 *   1 - ошибка связи
 */
char e24_reinit(void){
  int num;
  struct dataexchange_t pdata;
  hiddevice_t dev = HidOpenDevice(0x16C0, 0x05DF, L"COKPOWEHEU", L"e24 HID");
  if(!HidIsConnected(dev))return 1;
  pdata.cmd = CMD_REINIT;
  num = HidSendArr(dev, &pdata, sizeof(pdata));
  if(num != sizeof(pdata))return 1;
  HidCloseDevice(dev);
  return 0;
}
/* Конфигурация одного канала
 * 
 * Входные параметры:
 *   chan - номер канала
 *   freq - частота (отсчетов в секунду)
 *   adc_range - максимальное напряжение на канале (В). Будет подобрано ближайшее значение из доступных
 *   calibrate - калибровка:
 *     0 - нет
 *     1 - самокалибровка (default)
 *     2 - калибровка нуля
 *     3 - калибровка масштаба
 *     4 - смешанная калибровка
 *     5 - фоновая
 *     6 - внутренняя калибровка нуля
 *     7 - внутренняя калибровка масштаба
 *   subchannel - мультиплексор канала
 *     0 - канал A
 *     1 - канал B
 *     2 - заземление
 *     3 - тестовый режим
 * Результат:
 *   0 - успех
 *   1 - ошибка связи
 *   2 - неверно задан канал
 *   3 - невозможно передать частоту (ошибка связи)
 *   4 - невозможно передать диапазон (ошибка связи)
 *   5 - невозможно передать калибровку (ошибка связи)
 *   6 - невозможно передать subchannel (ошибка связи)
 *   7 - невозможно применить изменения (ошибка связи)
 */
char e24_configure(uint8_t chan, uint16_t freq, float adc_range, uint8_t calibrate, uint8_t subchannel){
  uint8_t mask;
  uint8_t adcr_res = 0;
  int num;
  struct dataexchange_t pdata;
  hiddevice_t dev;
  switch(chan){
    case 0: mask = (1<<0); break;
    case 1: mask = (1<<1); break;
    case 2: mask = (1<<2); break;
    case 3: mask = (1<<3); break;
    default: return 2;
  }
  dev = HidOpenDevice(0x16C0, 0x05DF, L"COKPOWEHEU", L"e24 HID");
  if(!HidIsConnected(dev))return 1;
  //freq
  if(freq != 0){
    pdata.cmd = CMD_ADC_FREQ;
    pdata.data2 = mask;
    pdata.data1 = (uint8_t)((freq>>8)&0xFF);
    pdata.data0 = (uint8_t)(freq & 0xFF);
    num = HidSendArr(dev, &pdata, sizeof(pdata));
    if(num != sizeof(pdata))return 3;
  }
  //adc_range
  if(adc_range != 0){
    adc_range = E24_MAX_RANGE / adc_range;
    if(adc_range < 2)adcr_res = 0;else
      if(adc_range < 4)adcr_res = 1; else
      if(adc_range < 8)adcr_res = 2; else
      if(adc_range < 16)adcr_res = 3; else
      if(adc_range < 32)adcr_res = 4; else
      if(adc_range < 64)adcr_res = 5; else
      if(adc_range < 128)adcr_res= 6; else
      adcr_res = 7;
    pdata.cmd = CMD_AMPL;
    pdata.data0 = adcr_res;
    num = HidSendArr(dev, &pdata, sizeof(pdata));
    if(num != sizeof(pdata))return 4;
  }
  //calibrate
  if(calibrate < 8){
    calibrate <<= 4;
    calibrate &= 0x70;
    pdata.cmd = CMD_CAL;
    pdata.data0 = calibrate;
    num = HidSendArr(dev, &pdata, sizeof(pdata));
    if(num != sizeof(pdata))return 5;
  }
  //subchannel
  if(subchannel < 4){
    pdata.cmd = CMD_SUBCHANNEL;
    pdata.data0 = subchannel;
    num = HidSendArr(dev, &pdata, sizeof(pdata));
    if(num != sizeof(pdata))return 6;
  }
  pdata.cmd = CMD_APPLY;
  num = HidSendArr(dev, &pdata, sizeof(pdata));
  if(num != sizeof(pdata))return 7;
  return 0;
}
/* Чтение выбранного канала с драйвера. Не напрямую с устройства!
 * 
 * Входные параметры:
 *   номер канала (1 - 4)
 * Результат:
 *   напряжение на канале (в вольтах) либо ошибка (1e10 В - в штатном режиме такое значение невозможно)
 */
double e24_read(uint8_t chan){
  double res;
  struct memfile_t pf;
  char filename[] = FILE_PATH"adc1.adc";
  if(chan > 4 || chan == 0)return 1e10;
  filename[sizeof(filename)-6]='0'+chan;
  if(memfile_create(filename, MEMFILE_READ, 100, &pf)!=MEMFILE_OK){
    memfile_close(&pf);
    return -1e10;
  }
  sscanf(pf.data,"%lf",&res);
  memfile_close(&pf);
  return res;
}
/*   Вывод подсказки. В основном для LabView. Поскольку там нет нормального вывода в консоль, приходится
 * возвращать строку.
 * 
 * Входные параметры:
 *   нет
 * Результат:
 *   строка с кратким описанием функций
 */
char* e24_help(){
  static const char helpstr[]=\
  "Library that configures e24_hid device\n"
  "To read ADC values from e24 use special user-space driver\n"
  "for read ADC values read files\n"
  "/home/dev/adc1.adc - /home/dev/adc4.adc or\n"
  "C:\\dev\\adc1.adc - C:\\dev\\adc4.adc\n"
  "1st value is voltage (in volts) on selected channel\n"
  "2nd value is time between starting driver and last reading value from e24\n"
  "To configure e24 use following functions placed in this library\n"
  "\n"
  "char e24_reinit(void)\n"
  "  hard reset and initialization e24\n"
  "  return value: 0 - failed (device is not connected) ; 1 - success\n"
  "char e24_configure(uint8_t chan, uint16_t freq, float adc_range, uint8_t calibrate, uint8_t subchannel)\n"
  "  chan - # channel (0 - 3)\n"
  "  freq - frequency (Hz) ; 0 - do not change\n"
  "  adc_range - set maximum voltage on channel (V) 0 - do not change\n"
  "  calibrate - calibration mode:\n"
  "    0 - none\n"
  "    1 - auto (default)\n"
  "    2 - ext calibration of zero\n"
  "    3 - ext calibration of range\n"
  "    4 - mixed calibration\n"
  "    5 - background calibration\n"
  "    6 - internal calibration of zero\n"
  "    7 - internal calibration of range\n"
  "    other values - do not change\n"
  "  subchannel:\n"
  "    0 - A\n"
  "    1 - B\n"
  "    2 - GND (or internal reference voltage?)\n"
  "    3 - test\n"
  "    other values - do not change\n"
  "  return value:\n"
  "    0 - OK\n"
  "    1 - failed open device\n"
  "    2 - wrong channel (must be 0 - 3)\n"
  "    3 - can not set frequency\n"
  "    4 - can not set range\n"
  "    5 - can not set calibration\n"
  "    6 - can not set subchannel\n"
  "    7 - can not aplly changes\n";
  return (char*)helpstr;
}
